# 16S rRNA Sequencing

16S rRNA sequencing and the associated analysis tools are a widely deployed method for identification, quantification, and classification of complex microbial samples taken from a wide variety of environments, from streams to stomachs. It is employed by environmental scientists, biologists, archaeologists, epidemiologists, and others to understand the complex relationship between microbes and the world around us. Earlham has a large collection of locally curated 16S sequence data to support this unit, along with installations of Qiime2, the open source toolchain we use for 16S rRNA analysis. 

There are many applications of 16S rRNA sequence analyis, that is many places you can find microbes in the world. This unit looks at microbes found in soil. It is composed of two major parts, first you will use a modest sized data set with the tutorial for Qiime2. Once you have worked-through that analysis and generated accurate visualizations you will work with a larger data set from Iceland, specifically soil samples taken at the Skalanes Nature Preserve. The Skalanes data set is large enough that you will need to embed the workflow in a job submission script for Slurm, the job scheduler. This will enable you to run it detached from a terminal and receive email when it is complete.

# Learning Goals

By the end of this unit, you will be able to dive into student-faculty research employing 16S analysis in a variety of contexts. 

1. Understand the different types of DNA sequencing that is done to support answering different types of questions.
1. Learn how to use Qiime2 for 16S rRNA sequence analysis
1. Understand paired end reads, joining, and filtering based on quality scores. 
1. Understand how species identification is done with 16S databases. 
1. Develop your command line data processing skills, e.g. Slurm and other forms of automation with Bash. 

# Resources

These materials will give you an introduction to the type of sequencing and analysis done on 16S rRNA. 

## Overviews and journal articles 

1. [DNA Sequencing](https://en.wikipedia.org/wiki/DNA_sequencing) on Wikipedia - introduction, sections 1, 2
1. [16S rRNA sequencing](https://en.wikipedia.org/wiki/16S_ribosomal_RNA) on Wikipedia - introduction, sections 1, 2, 3, 4, 5, and 6 (through 16S ribosomal databases)
1. [Environmental DNA Analysis](https://www.illumina.com/techniques/sequencing/dna-sequencing/targeted-resequencing/environmental-dna.html)
1. [Ancient and modern environmental DNA](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4275890/) 
1. [FASTQ sequence format description](https://en.wikipedia.org/wiki/FASTQ_format) on Wikipedia
1. [Selections from Hunter on Molecular Biology](http://tandy.cs.illinois.edu/Hunter_MolecularBiology.pdf): 1, 1.1, 1.2; 2, 2.1, 2.2; 4, 4.1, 4.3; 5.1, 5.2; 7 Gel Electrophoresis, Gene Mapping and Sequencing, 7.1 

## Software Packages 

The compute server pollock.cluster.earlham.edu has all the software listed below installed. Contact the CS admins if you need a cluster username to complete this unit. 

1. [A highlevel overview of Qiime2](https://www.youtube.com/playlist?list=PLbVDKwGpb3XmkQmoBy1wh3QfWlWdn_pTT) - watch this first
1. [Qiime2 tutorial](https://docs.qiime2.org/2020.8/tutorials/moving-pictures/) - complete this tutorial next
1. [Qiime2 online visualization tool](https://view.qiime2.org/) - you can scp the output from Qiime2 (.qva/.qvz files) to your local machine and then upload to this interface. (Support for viewing artifacts directly from the servers is coming soon.)

## Iceland Data and Workflow

### Data

These samples were taken at two locations in Iceland during the summer of 2014 by a group of Earlham students and faculty. Most are from [Skalanes](http://skalanes.com) (65.287, -13.690), the research center where we live and work in the Eastfjords, the remaining one is from an unusual geological feature near the Godafoss waterfall in the North-east part of the island. Skalanes is home to avian nesting grounds (including the Arctic Tern, a super long-distance commuter), archaeological sites dating back to the early settlement of Iceland, and a variety of sustainability projects and reforestation projects. 

The "kit" for this portion of the module is composed of the following:
1. [Meet the Data](meet-the-data-Iceland2014.pdf) (PDF)
1. [Metadata](metadata-Iceland2014.csv) (CSV)
1. /cluster/metagenomes/Iceland-2014/sequences/ (FASTQ) - The filename includes the Spot ID, it is the two digit integer that follows "CP-". This matches the Spot IDs in Meet the Data and in the metadata. Remember Bash's token editing capabilities if you want to bulk rename the data files for your workflow. These are paired-end reads with an R1 and R2 file for each sample. 

### Workflow

This assumes you are running Qiime2 on pollock.cluster.earlham.edu. Using Qiime2's Moving Pictures tutorial as a pattern build a new workflow (probably in a Bash script) for the Skalanes 2014 data. You can use [these notes](https://docs.google.com/document/d/1y_FL7uQ7j5sUYQaAGtXzNVz-pbFHe-8EXY4OKRMwRq0/edit#heading=h.zasd3t577txk) as a reference.


# Deliverables

1. Visualizations from the Qiime2 tutorial's data set
1. Visualizations from the Iceland 2014 data set 
1. How can you improve the performance (i.e. reduce wall time) for the analysis of the Skalanes 2019 data set? Be prepared to disucss this at your last weekly meeting for the course.

# Optional Activities 

1. Install qiime2 on your personal machine. This will enable you to run large analysis on pollock.cluster.e.e and then scp the .qva/.qvz files to your machine for further analysis & visualization. 


