# Whole genome sequencing

Whole genome sequencing (WGS) and analysis supports the study of both the species and individual members of a species from both the mitochondrial and nuclear DNA perspectives. One use of this approach is to study DNA extracted from soil or bone to identify the species and to "date" the fragments. This allows the samples to be placed in context with environmental, archaeological, and biological data. Earlham has a large collection of locally curated whole genome data to support this unit, along with installations of BowTie, SamTools, and the other components of a WGS analysis workflow. 

# Learning Goals

By the end of this unit, you will be able to dive into student-faculty research using whole genome sequence analysis. 

1. Understand the different types of DNA sequencing that is done to support answering different types of questions.
1. Understand the output from an Illumina HiSeq 2500 sequencer platform (version 1.8, Phred33)
1. Understand paired end reads, joining, and filtering based on quality scores. 
1. Understand alignment to a dictionary for species identification and how to use bowtie to align a set of sequencer output. 
1. Learn samtools for analyzing and summarizing the alignment maps generated by bowtie.
1. Develop your command line data processing skills, e.g. nested Bash loops over files and parameters and advanced filtering with egrep.

# Resources

These materials will give you an introduction to shotgun sequencing as it is used for DNA recovered from the environment.

## Overviews and journal articles 

1. [DNA Sequencing](https://en.wikipedia.org/wiki/DNA_sequencing) on Wikipedia - introduction, sections 1, 2
1. [Shotgun sequencing](https://en.wikipedia.org/wiki/Shotgun_sequencing) on Wikipedia - introduction, sections 1, 2
1. [Environmental DNA Analysis](https://www.illumina.com/techniques/sequencing/dna-sequencing/targeted-resequencing/environmental-dna.html)
1. [Ancient and modern environmental DNA](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4275890/) 
1. [FASTQ sequence format description](https://en.wikipedia.org/wiki/FASTQ_format) on Wikipedia

## Data

1. Stod data (archeaological site, 64.8428, -13.9653): 
- sequence data - /cluster/fieldscience/Iceland-2018/Stod/aDNA/cs266-data or http://cluster.earlham.edu/fieldscience/Data/Iceland-2018/Stod/aDNA/cs266-data/
- metadata (sample spot geocodes, images, soil parameters) - URL 

## Software Packages 

The compute server pollock.cluster.earlham.edu has all the software listed below installed.

1. [AdapterRemoval](https://github.com/MikkelSchubert/adapterremoval)
1. [bowtie overview](https://en.wikipedia.org/wiki/Bowtie_(sequence_analysis)) on Wikipedia
1. [bowtie tutorial](http://bowtie-bio.sourceforge.net/tutorial.shtml) on SourceForge
1. [samtools overview and examples](http://www.htslib.org/doc/samtools.html)
1. egrep - `man egrep`

## Example Workflow

This assumes you are running on pollock.cluster.earlham.edu and starting with the small version of the FASTQ files. Always start with a subset of the data and test the workflow from end-to-end before moving on to the large and jumbo sizes. Remeber to adapt thesecommand examples to the specific filenames, etc.

1. The FASTQ output from the sequencing core is in v1.8 format using a Phred33 quality score encoding. eautils was used to determine the adapter seqences and build a list of them (adapters.list). AdapterRemoval is used to remove the adapters, join the R1 and R2 reads, and perform basic quality filtering. 
- `$ /usr/local/bin/AdapterRemoval --trimqualities --collapse --minalignmentlength 11 --minlength 25 --minquality 25 --trimns --file1 L2Sample-small-R1.fastq --file2 L2Sample-small-R2.fastq --qualitybase 33 --qualitymax 73 --basename L2Sample-small --outputcollapsed L2Sample-small.fastq --adapter-list adapters.list`
2. Once the reads have been joined and filtered species identification (often called alignment) can be done using one of the databases that are available from the NCBI. For this example we will use the mitochondrial DNA taxid databases. 
- `$ /usr/local/bin/bowtie2 -q --phred33 --local --very-sensitive --no-unal -k 2 --dovetail -x /mounts/pollock/aDNA/reference-genomes/getLCA/mitochondrion.genomic.taxid.fasta -U L2Sample-small.fastq > L2Sample-small.sam`
3. The output from bowtie is a sequence alignment map (SAM). These are often converted to binary alignment map (BAM) format to improve performance before they are analyzed. samtools is a powerful utility that works with alignment maps in both formats. 
- `$ module load samtools`
- `$ # convert the SAM file to a BAM file`
- `$ samtools view -b -S -o L2Sample-small.bam L2Sample-small.sam`
- `$ # count the number of alignments in the map`
- `$ samtools view -h L2Sample-small.bam | wc -l`

# Deliverables

1. Command line that demonstrates how to align Stod sequence data to the NCBI mitochondrial database using bowtie
1. Command line that demonstrates how to filter all of the best matches for species XXX from the alignment maps generated by bowtie.
1. How can you improve the performance of removing the adapters and aligning the sequences? 
